/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <malloc.h>
#include <stdio.h>
#include <strings.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"
#include "Index.h"
#include <cxxcls/vdiction.h>

////////////////////////////////////////////////////////////////////////////
//
// Document Methods
//
////////////////////////////////////////////////////////////////////////////

extern o_4b	debugMode;

// JGM 93/10/15 Added space between '>' tokens to avoid cfront 3.0
// warnings - next 2 lines

Link<VEIDictionary<o_4b, Document> >	documentIndex;
 
Implement PClassObject<VEIDictionary<o_4b, Document> >;
Implement PClassObject<Document>;

////////////////////////////////////////////////////////////////////////////
//
// Document Constructor
//
////////////////////////////////////////////////////////////////////////////

Document::Document(o_4b cpId, Link<CompositePart> cp)
		 : id(cpId), part(cp), title(TitleSize), text(DocumentSize)
{
    o_4b        stringLen;
    o_4b        curChar, strChar;
    char*	str;
    char	myText[DocTextLength];

    // prepare and fill in the document title
    title.set_length(TitleSize);
    sprintf((char*)title, "Composite Part %08d", cpId);
    if (debugMode) {
        printf("Document::Document(title = %s)\n", (char*)title);
    }

    sprintf(myText, DocumentText, cpId);
    stringLen = strlen(DocumentText);

    curChar = strChar = 0;
    // JGM 15/10/93 Added cast to prevent compiler warnings
    text.set_length((int)DocumentSize);
    str = (char*)text;
    while (curChar < DocumentSize - 1) {
	str[curChar++] = myText[strChar++];
	if (strChar >= stringLen) { strChar = 0; }
    }
    str[curChar] = '\0';
    textSize = DocumentSize;
    textLen = DocumentSize;
    if (documentIndex == NULL_LINK) {
	Index	docIndex("document index");

	documentIndex = docIndex.getIndex();
    }
    documentIndex->add(cpId, *this);
}

// destructor
Document::~Document()
{
    if (documentIndex == NULL_LINK) {
	Index	docIndex("document index");

	documentIndex = docIndex.getIndex();
    }
    documentIndex->remove(id);
}

////////////////////////////////////////////////////////////////////////////
//
// Document searchText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

o_4b Document::searchText(const char c)
{
    if (debugMode) {
        printf("                    Document::searchText(title = %s)\n", title);
    }

    // count occurrences of the indicated letter (for busy work)
    o_4b	count = 0;
    char*	str = (char*)text;

    while (*str != '\0') {
	if (*str++ == c) { count++; }
    }

    if (debugMode) {
        printf("                    [found %d '%c's among %d characters]\n",
	        		     count, c, str - (char*)text);
    }
    return count;
}


////////////////////////////////////////////////////////////////////////////
//
// Document replaceText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

o_4b Document::replaceText(char *oldString, char *newString)
{
    if (debugMode) {
        printf("                    Document::changeText(title = %s)\n", title);
    }

    // check to see if the text starts with the old string

    o_4b	oldTextLength = textLen;
    o_4b	oldStrLength  = strlen(oldString);
    o_bool	foundMatch;

    foundMatch = strncmp((char *)text, oldString, (int)oldStrLength) == 0;
    // if so, change it to start with the new string instead
    if (foundMatch)
    {
        o_4b	newStrLength = strlen(newString);
        o_4b	lengthDiff = newStrLength - oldStrLength;

	dirty();
	if (lengthDiff > 0)
	{
	    textSize = newStrLength + 1;
	    // JGM 15/10/93 Added cast to prevent compiler warnings
	    text.set_length((int)textSize);
	}
	textLen = newStrLength;
	// JGM 15/10/93 Added (unsigned) cast to prevent compiler warnings
	strncpy((char*)text, newString, (unsigned)newStrLength);
    }

    if (debugMode) 
    {
	if (foundMatch) {
            printf("                    [changed \"%s\" to \"%s\"]\n",
	                                 oldString, newString);
	} else {
            printf("                    [no match, so no change was made]\n");
	}
    }

    return foundMatch;
}
