/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #4 - randomly choose "Query4RepeatCnt" document objects by 
// lookup on their title field.  An index can be used for the
// actual lookup.  
//
//
/////////////////////////////////////////////////////////////////

extern o_4b	debugMode;

static char	title[TitleSize];

//////////////////////////////////////////////////////////////////
//
// Routine to do Query #4
//
//////////////////////////////////////////////////////////////////

o_4b  query4()
{
    // now randomly select a document via its title index 
    // and trace a path to the base assemblies that use 
    // its associated composite part (and then
    // repeat this process the desired number of times)

    // set random seed so "hot" runs are truly hot
    srandom(1);

    o_4b	count = 0;

    for (o_word i = 0; i < Query4RepeatCnt; i++) 
    {
	// generate random document title and lookup document
	o_4b	compPartId = (o_4b)(random() % TotalCompParts) + 1;

        sprintf(title, "Composite Part %08d", compPartId);

	LinkVstr<Document>	documents;
	Document*		document;
	
	documents = PClassObject<Document>::Object().select(NULL, FALSE,
			PAttribute("Document::title") == title);
	if ( documents.is_empty() )
	{
            fprintf(stderr, "ERROR: Unable to find document called \"%s\"\n",
	            title);
            exit(1);
        }

        if (debugMode) {
            printf("    In Query4, document title = \"%s\"\n", title);
        }

	document = documents[0];
	documents.release();

	// now walk back up the path(s) to the associated base assemblies
	// (based on private uses of the composite part, at least for now)

	LinkVstr<BaseAssembly>&	bases = document->part->usedInPriv;
	o_4b			size = bases.size();

	// establish baseassembly iterator
	for (o_word j = 0; j < size; j++ ) {
	    bases[j]->DoNothing();
	    count++;
	}
    }
    return count;
}
