/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #6 - find all assemblies (base or complex) B that "use" 
// (directly or transitively ) a composite part with a more recent 
// build date than B's build date.
//
//
/////////////////////////////////////////////////////////////////

extern o_4b	debugMode;

//////////////////////////////////////////////////////////////////
//
// Recursive Workhorse Routine for Query #6
//
//////////////////////////////////////////////////////////////////

o_4b checkMaxCompDate(Assembly* assembly, o_4b& count)
{
    // check to see if this assembly uses a more recently dated composite
    // part, and process it if so - handling complex and base assemblies
    // differently since the former have subassemblies while the latter
    // directly use composite parts

    o_4b maxSubDate = -1;

    if (assembly->assmType == Complex)
    {
        // complex assembly case - check its subassemblies recursively
        if (debugMode) {
            printf("In Query6:  complex assembly %d, buildDate = %d\n",
                    assembly->id, assembly->buildDate);
        }

	ComplexAssembly*	complexAssembly = (ComplexAssembly*)assembly;
        LinkVstr<Assembly>&	assemblies = complexAssembly->subAssemblies;
	o_4b			asize = assemblies.size();

	for (o_word i = 0; i < asize; i++) {
	    o_4b		rsltDate;

	    rsltDate = checkMaxCompDate(assemblies[i], count);
	    if (rsltDate > maxSubDate)
		maxSubDate = rsltDate;
	}
    } 
    else 
    {
        // base assembly case - check its composite part build dates
        if (debugMode) {
            printf("In Query6:  base assembly %d, buildDate = %d\n",
                    assembly->id, assembly->buildDate);
            printf("    composite part dates = { ");
        }

	BaseAssembly*			baseAssembly  = (BaseAssembly*)assembly;
        LinkVstr<CompositePart>&	compositeParts =
						baseAssembly->ComponentsPriv;
	o_4b				csize = compositeParts.size();

	for ( o_word i = 0; i < csize; i++) {
	    CompositePart*	compositePart = compositeParts[i];

            if (debugMode) { printf("%d ",compositePart->buildDate); }
	    if (compositePart->buildDate > maxSubDate)
		maxSubDate = compositePart->buildDate;
        }
        if (debugMode) { printf("}\n"); }
    }

    // see if the maximum build date of composite parts used by this assembly
    // is greater than the assembly's build date, and process the assembly if
    // this is found to be the case;  also, return the maximum build date for
    // this subassembly

    if (maxSubDate > assembly->buildDate) {
        assembly->DoNothing();
	count++;
    }
    return maxSubDate;
}

//////////////////////////////////////////////////////////////////
//
// Top-Level Routine to do Query #6
//
//////////////////////////////////////////////////////////////////


o_4b query6()
{
    // scan modules, visiting all of the assemblies in each one, checking
    // to see if they use (either directly or indirectly) a more recently
    // dated composite part;  the visits are done by recursively traversing
    // the assemblies, checking them on the way back up (to avoid lots of
    // repeated work)

    // defaults to full scan
    LinkVstr<Module>	modules = PClassObject<Module>::Object().select(
					NULL, FALSE, NULL_PREDICATE);
    o_4b		size = modules.size();
    o_4b		count = 0;

    for ( o_word i = 0; i < size; i++) {
	o_4b	rslt = checkMaxCompDate(modules[i]->designRoot, count);
    }
    modules.release();
    return count;
}
